package com.persagy.dmp.storage.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.file.client.FileClientFacade;
import com.persagy.dmp.file.client.FileMd5ClientFacade;
import com.persagy.dmp.file.model.FileInfo;
import com.persagy.dmp.file.model.FileInfoCreator;
import com.persagy.dmp.file.model.FileMd5;
import com.persagy.dmp.file.model.FileMd5Creator;
import com.persagy.dmp.storage.constant.FileCommonConst;
import com.persagy.dmp.storage.service.FileStorageFactory;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.util.List;

/**
 * 文件存储助手
 * @author Charlie Yu
 * @date 2021-05-15
 */
@Slf4j
public class FileStorageHelper {

    /**
     * 上传文件
     * @param fileInfo
     * @param in
     * @return
     */
    public static String uploadFile(FileInfo fileInfo, InputStream in) {
        // 1.获得文件大小
        fileInfo.setFileSize(getStreamSize(in));
        // 2.获得文件的md5值
        String fileMd5Str = DigestUtil.md5Hex(in);
        FileMd5 fileMd5 = FileMd5ClientFacade.loadByMd5(fileMd5Str);
        if (null!=fileMd5
                && FileCommonConst.UPLOAD_PART.equals(fileMd5.getUploadStatus())){
            throw new BusinessException(ResponseCode.B0300.getCode(),ResponseCode.B0300.getDesc());
        }
        // 3.创建新的存储的fileInfo
        FileInfo saveFileInfo = FileInfoCreator.of(fileInfo.getGroupCode(), fileInfo.getBusinessId(), fileInfo.getFileName(),
                fileMd5Str, fileInfo.getExpireDate());
        // 秒传
        saveFileInfo = FileClientFacade.insert(saveFileInfo);
        if (null!=fileMd5
                && FileCommonConst.UPLOAD_SUCCESS.equals(fileMd5.getUploadStatus())){
            return saveFileInfo.getId();
        }
        // 4.没上传过
        fileMd5 = FileMd5Creator.of(fileInfo.getFileBucket(), fileMd5Str, FileCommonConst.UPLOAD_SUCCESS);
        fileMd5.setFileName(fileInfo.getFileName());
        fileMd5.setGroupCode(fileInfo.getGroupCode());
        fileMd5.setFileSize(fileInfo.getFileSize());
        fileMd5 = FileMd5ClientFacade.insert(fileMd5);
        FileStorageFactory.getService().upload(fileMd5.getFileBucket(), fileMd5.getFilePath(), in);
        return saveFileInfo.getId();
    }

    /**
     * 上传图片
     * @param fileInfo
     * @param in
     * @return
     */
    public static String uploadImage(FileInfo fileInfo, InputStream in) {
        // 图片压缩
        InputStream cutIn = ImageHelper.cutImage(in);
        return uploadFile(fileInfo, cutIn);
    }

    /**
     * 修改文件
     * @param fileId
     * @param fileName
     * @param in
     * @return
     */
    public static String replaceFile(String fileId, String fileName, InputStream in) {
        FileInfo fileInfo = FileClientFacade.load(fileId);
        // 先删除文件
        FileStorageFactory.getService().delete(fileInfo.getFileBucket(), fileInfo.getFilePath());
        // 更新信息后上传
        fileInfo.setFileName(fileName);
        fileInfo.setFileSize(getStreamSize(in));
        fileInfo = FileClientFacade.update(fileInfo);
        FileStorageFactory.getService().upload(fileInfo.getFileBucket(), fileInfo.getFilePath(), in);
        return fileInfo.getId();
    }

    /**
     * 更新图片
     * @param fileId
     * @param fileName
     * @param in
     * @return
     */
    public static String replaceImage(String fileId, String fileName, InputStream in) {
        // 图片压缩
        InputStream cutIn = ImageHelper.cutImage(in);
        return replaceFile(fileId, fileName, cutIn);
    }

    /**
     * 删除文件及图片
     * @param fileId
     */
    public static void deleteFile(String fileId) {
        FileInfo fileInfo = FileClientFacade.load(fileId);
        List<FileInfo> fileInfos = FileClientFacade.queryByFileMd5(fileInfo.getFileMd5());
        if (CollUtil.isNotEmpty(fileInfos) && fileInfos.size()>1){
            // 多余一个绑定到相同的md5文件上,仅删除文件信息即可
            FileClientFacade.delete(fileId);
            return;
        }
        // 只有一个绑定到相同的md5文件上,删除文件信息,文件MD5信息,文件信息
        FileMd5 fileMd5 = FileMd5ClientFacade.loadByMd5(fileInfo.getFileMd5());
        FileClientFacade.delete(fileId);
        FileMd5ClientFacade.delete(fileMd5.getId());
        FileStorageFactory.getService().delete(fileMd5.getFileBucket(), fileMd5.getFilePath());
    }

    /**
     * 下载
     * @param fileId 文件id
     * @return 文件下载地址
     */
    public static String downloadUrl(String fileId) {
        FileInfo fileInfo = FileClientFacade.load(fileId);
        FileMd5 fileMd5 = FileMd5ClientFacade.loadByMd5(fileInfo.getFileMd5());
        return FileStorageFactory.getService().fetchUrl(fileMd5.getFileBucket(), fileMd5.getFilePath());
    }

    /**
     * 下载
     * @param fileId 文件id
     * @return 文件流
     */
    public static InputStream downloadStream(String fileId) {
        FileInfo fileInfo = FileClientFacade.load(fileId);
        FileMd5 fileMd5 = FileMd5ClientFacade.loadByMd5(fileInfo.getFileMd5());
        return FileStorageFactory.getService().download(fileMd5.getFileBucket(), fileMd5.getFilePath());
    }

    /**
     * 下载
     * @param fileId 文件id
     * @return 文件内容
     */
    public static byte[] downloadContent(String fileId) {
        // TODO 暂不提供，依赖前端显示规则
        return null;
    }

    /**
     * 获取文件大小
     * @param stream
     * @return
     */
    private static Long getStreamSize(InputStream stream) {
        try {
            return Long.valueOf(stream.available());
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new BusinessException("文件上传失败！");
        }
    }
}
